import {
  Flex,
  Text,
  HStack,
  useToast,
  ToastPositionWithLogical,
} from "@chakra-ui/react";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import React, { useEffect, useState } from "react";

import { Task } from "../../model/taskModel";
import { User } from "../../model/userModel";
import {
  deleteTask,
  joinTask,
  completeTask,
  leaveTask,
} from "../../services/taskService";
import { getLoggedUserFromLocalStorage } from "../../utils/utils";
import Button from "../Button";
import EvaluationModal from "../EvaluationModal";

interface IProps {
  eventId: string;
  task: Task;
  eventType: string | undefined;
  eventOwner: User | undefined;
  findEvent: () => void;
}

const TaskCard = ({
  eventId,
  task,
  eventType,
  eventOwner,
  findEvent,
}: IProps): React.ReactElement => {
  const [loggedUser, setLoggedUser] = useState<User>();
  const [loading, setLoading] = useState(false);
  const [openEvaluationModal, setOpenEvaluationModal] = useState(false);
  const { id, description, endDate, volunteer, owner } = task;

  const toast = useToast();

  const loggedUserIsEventOwner = eventOwner?.id === loggedUser?.id;
  const loggedUserIsVoluntary = volunteer && volunteer.id === loggedUser?.id;
  const loggedUserIsOwner = owner && owner.id === loggedUser?.id;
  const isActionEvent = eventType === "ACTION";
  const textColor = endDate ? "white" : "blue.200";

  useEffect(() => {
    (async () => {
      setLoggedUser(getLoggedUserFromLocalStorage());
    })();
  }, []);

  const showToastMessage = (
    title: string,
    status: "info" | "warning" | "success" | "error" | undefined,
    position: ToastPositionWithLogical | undefined
  ) => {
    toast({
      title,
      status,
      position,
      isClosable: true,
    });
  };

  const deleteEventTask = async () => {
    setLoading(true);
    try {
      await deleteTask(eventId, id);
      findEvent();
      showToastMessage("Tarefa deletada com sucesso!", "success", "top-right");
    } catch (e: any) {
      showToastMessage(e, "error", "top-right");
    }
    setLoading(false);
  };

  const joinEventTask = async () => {
    setLoading(true);
    try {
      await joinTask(eventId, id);
      findEvent();
      showToastMessage(
        "Participação efetuada com sucesso!",
        "success",
        "top-right"
      );
    } catch (e: any) {
      showToastMessage(e, "error", "top-right");
    }
    setLoading(false);
  };

  const concludeEventTask = async () => {
    setLoading(true);
    try {
      await completeTask(eventId, id);
      findEvent();
      showToastMessage("Tarefa concluída com sucesso!", "success", "top-right");
      setOpenEvaluationModal(true);
    } catch (e: any) {
      showToastMessage(e, "error", "top-right");
    }
    setLoading(false);
  };

  const leaveEventTask = async () => {
    setLoading(true);
    try {
      await leaveTask(eventId, id);
      findEvent();
      showToastMessage("Participação cancelada!", "success", "top-right");
    } catch (e: any) {
      showToastMessage(e, "error", "top-right");
    }
    setLoading(false);
  };

  return (
    <Flex
      w="90%"
      h="150px"
      flexDir="column"
      justifyContent="space-evenly"
      alignItems="center"
      border="1px solid"
      borderColor="blue.200"
      borderRadius="30px 0 0 0"
      boxSizing="border-box"
      p="10px"
      bgColor={endDate ? "blue.200" : "white"}
      opacity={endDate ? 0.7 : 1}
      boxShadow="lg"
    >
      <Flex w="100%" h="20%" justifyContent="space-between">
        {volunteer ? (
          <Text fontSize="14px" color={textColor}>
            <b>Voluntário:</b> {volunteer.name}
          </Text>
        ) : (
          <Text fontSize="14px" color={textColor}>
            <b>Seja o voluntário dessa tarefa!</b>
          </Text>
        )}
        {endDate
          ? ""
          : (loggedUserIsOwner || loggedUserIsEventOwner) && (
              <Button
                theme="secondary"
                h="34px"
                w="34px"
                borderRadius="50%"
                bgColor="transparent"
                onClick={deleteEventTask}
                isDisabled={loading}
              >
                <FontAwesomeIcon
                  icon="trash"
                  color="#265DA2"
                  size="lg"
                  cursor="pointer"
                />
              </Button>
            )}
      </Flex>

      <Flex h="60%" justifyContent="center" alignItems="center">
        <Text fontSize="12px" color={textColor} wordBreak="break-word">
          {description}
        </Text>
      </Flex>

      {isActionEvent && (
        <HStack h="20%">
          {(endDate && (
            <Button
              theme="quaternary"
              w="140px"
              h="26px"
              fontSize="14px"
              bgColor="blue.500"
              isDisabled
            >
              <b>CONCLUÍDA</b>
            </Button>
          )) ||
            (loggedUserIsVoluntary && (
              <Button
                isLoading={loading}
                theme="quaternary"
                w="140px"
                h="26px"
                fontSize="14px"
                onClick={leaveEventTask}
              >
                <b>ABANDONAR</b>
              </Button>
            )) ||
            (!volunteer && (
              <Button
                isLoading={loading}
                theme="secondary"
                w="140px"
                h="26px"
                fontSize="14px"
                onClick={joinEventTask}
              >
                <b>PARTICIPAR</b>
              </Button>
            ))}
          {!endDate &&
            !!volunteer &&
            (!loggedUserIsVoluntary || loggedUserIsEventOwner) && (
              <Button
                isLoading={loading}
                theme="secondary"
                w="140px"
                h="26px"
                fontSize="14px"
                onClick={concludeEventTask}
              >
                <b>CONCLUIR</b>
              </Button>
            )}
        </HStack>
      )}

      {/* openEvaluationModal, setOpenEvaluationModal */}
      <EvaluationModal
        open={openEvaluationModal}
        eventId={eventId}
        taskId={id}
        close={() => setOpenEvaluationModal(false)}
      />
    </Flex>
  );
};

export default TaskCard;
