/* eslint-disable no-restricted-globals */
import {
  Box,
  Flex,
  Radio,
  RadioGroup,
  Text,
  ToastPositionWithLogical,
  useToast,
} from "@chakra-ui/react";
import React, { useEffect, useState } from "react";
import Moment from "react-moment";
import { useHistory, useParams, Link } from "react-router-dom";

import BottomMenu from "../../components/BottomMenu";
import Button from "../../components/Button";
import EventDetailLabel from "../../components/EventDetailLabel";
import { Header } from "../../components/Header";
import UserReasonModal from "../../components/UserReasonModal";
import { EventDetailsResponseDTO } from "../../model/eventModel";
import { donate, findEventDetails } from "../../services/eventService";

interface RouteParams {
  id: string;
}

interface ISelectedUser {
  id: string | undefined;
  description: string;
  name: string | undefined;
}

const AdminListEventDetails = (): React.ReactElement => {
  const [event, setEvent] = useState<EventDetailsResponseDTO>();
  const [loading, setLoading] = useState(false);
  const [openModalUserDetail, setOpenModalUserDetail] = useState(false);
  const [selectedUser, setSelectedUser] = useState<ISelectedUser>();
  const [donatedTaskId, setDonatedTaskId] = useState("");
  const [selectedTaskId, setSelectedTaskId] = React.useState("");

  const params = useParams<RouteParams>();
  const { id } = params;
  const history = useHistory();
  const toast = useToast();

  const findEvent = () => {
    (async () => {
      const response = await findEventDetails(id);
      setEvent(response.data);
      const responseData: EventDetailsResponseDTO = response.data;
      const taskDonated = responseData?.tasks?.filter((t) => t.donated);
      if (taskDonated && taskDonated.length > 0)
        setDonatedTaskId(taskDonated[0].id);
    })();
  };

  useEffect(() => {
    findEvent();
  }, []);

  const showToastMessage = (
    title: string,
    status: "info" | "warning" | "success" | "error" | undefined,
    position: ToastPositionWithLogical | undefined
  ) => {
    toast({
      title,
      status,
      position,
      isClosable: true,
    });
  };

  const modalUserDetails = (userId: string | undefined) => {
    if (!userId) return;
    const filtered = event?.tasks?.filter(
      (task) => task.volunteer.id === userId
    );
    if (filtered && filtered.length > 0) {
      const selected: ISelectedUser = {
        id: filtered[0].volunteer.id,
        description: filtered[0].description,
        name: filtered[0].volunteer.name,
      };
      setSelectedUser(selected);
      setOpenModalUserDetail(true);
    }
  };

  const donateProduct = async () => {
    setLoading(true);
    try {
      await donate(id, selectedTaskId);
      findEvent();
      showToastMessage("Doação concluída com sucesso!", "success", "top-right");
    } catch (e: any) {
      showToastMessage(e, "error", "top-right");
    }
    setLoading(false);
  };

  return (
    <>
      <Header variant="double" position="fixed" zIndex="99" />
      <Box
        d="flex"
        flexDirection="column"
        alignItems="center"
        justifyContent="space-evenly"
        h="calc(100vh - 50px)"
        overflowY="auto"
        pt={["40px", "100px"]}
        pb="10px"
      >
        <Box w="100%" p="5px">
          <Button
            theme="secondary"
            variant="back"
            onClick={history.goBack}
            _hover={{ background: "yellow.500" }}
            _active={{ background: "yellow.500" }}
          />

          <Flex
            w="100%"
            alignItems="center"
            justifyContent="center"
            direction="column"
          >
            <Text color="yellow.500" fontWeight="extrabold">
              SOLICITAÇÕES EM ABERTO:
            </Text>
          </Flex>
        </Box>
        <Box w="100%" p="16px">
          <Flex w="100%" alignItems="center">
            <Text fontWeight="bold">{event?.title}</Text>
            <Link
              style={{ color: "#265DA2", fontSize: "0.8rem" }}
              to={`/events/${id}`}
            >
              (ver detalhes)
            </Link>
          </Flex>
          <Flex justifyContent="space-between" marginTop="10px" width="100%">
            <EventDetailLabel
              variant="secondary"
              icon="calendar-alt"
              alignItems="center"
            >
              {!!event?.eventDate && (
                <Moment format="DD/MM/YYYY">{event?.eventDate}</Moment>
              )}
            </EventDetailLabel>
            <EventDetailLabel
              variant="secondary"
              icon="clock"
              alignItems="center"
              ml="5px"
            >
              {event?.eventTime?.substr(0, 5)}
            </EventDetailLabel>
            <EventDetailLabel
              variant="secondary"
              icon="map-marker-alt"
              alignItems="center"
              ml="5px"
              isTruncated
              maxWidth="50%"
              title={event?.street}
            >
              {event?.street}
            </EventDetailLabel>
          </Flex>
        </Box>
        <Flex direction="column" alignItems="center" h="60%">
          <Text color="yellow.500" fontWeight="bold">
            Participantes
          </Text>
          {event && event.tasks && event.tasks.length > 0 ? (
            <RadioGroup onChange={setSelectedTaskId} value={selectedTaskId}>
              <Flex w="100%" direction="column" alignItems="center">
                {event.tasks.map(
                  (task) =>
                    task?.volunteer?.id && (
                      <Radio
                        key={task?.volunteer?.id}
                        value={task?.id}
                        colorScheme="yellow"
                        // isDisabled={
                        //   donatedTaskId !== "" && donatedTaskId !== task.id
                        // }
                        // defaultChecked={task?.donated}
                      >
                        <Flex
                          h="28px"
                          w="80vw"
                          p="16px"
                          m="10px 0"
                          alignItems="center"
                          bgColor="blue.200"
                          borderRadius="5px"
                          justifyContent="left"
                          cursor="pointer"
                          onClick={() => modalUserDetails(task.volunteer.id)}
                        >
                          <Text color="white" defaultChecked>
                            {task?.volunteer?.name}
                          </Text>
                        </Flex>
                      </Radio>
                    )
                )}
              </Flex>
            </RadioGroup>
          ) : (
            <Text>Nenhum participante encontrado!</Text>
          )}
        </Flex>
        {donatedTaskId === "" && selectedTaskId !== "" && (
          <Flex justifyContent="center">
            <Button
              theme="secondary"
              onClick={donateProduct}
              isLoading={loading}
              _hover={{ background: "yellow.500" }}
              _active={{ background: "yellow.500" }}
            >
              DOAR
            </Button>
          </Flex>
        )}
      </Box>
      {selectedUser && (
        <UserReasonModal
          open={openModalUserDetail}
          user={selectedUser}
          close={() => setOpenModalUserDetail(false)}
        />
      )}
      <BottomMenu defaultIndex={0} />
    </>
  );
};
export default AdminListEventDetails;
