import { Box, Img, Image, Text } from "@chakra-ui/react";
import React, { useContext, useEffect, useState } from "react";
import { useHistory } from "react-router-dom";

import defaultImage from "../../assets/default-event-image.png";
import logoImage from "../../assets/logo-1X.png";
import BottomMenu from "../../components/BottomMenu";
import EventFilter from "../../components/EventFilter";
import EventInfo from "../../components/EventInfo";
import { Header } from "../../components/Header";
import ImageName from "../../components/ImageName";
import SearchBar from "../../components/SearchBar";
import { FilterContext } from "../../contexts/FilterContext";
import { EventResponseListDTO } from "../../model/eventModel";
import { User } from "../../model/userModel";
import { findActiveEvent } from "../../services/eventService";
import { getLoggedUserFromLocalStorage } from "../../utils/utils";

const ListEvent = (): React.ReactElement => {
  const [events, setEvents] = useState<Array<EventResponseListDTO>>([]);
  const [filteredEvents, setFilteredEvents] =
    useState<EventResponseListDTO[]>(events);
  const [search, setSearch] = React.useState("");
  const [loggedUser, setLoggedUser] = useState<User | undefined>(undefined);

  const { filters } = useContext(FilterContext);

  const history = useHistory();

  useEffect(() => {
    (async () => {
      setLoggedUser(getLoggedUserFromLocalStorage());
      const response = await findActiveEvent();
      const foundEvents = response;

      setEvents(foundEvents);
      setFilteredEvents(foundEvents);
    })();
  }, []);

  const updateFilters = () => {
    const filtered = events.filter((event) => {
      const eventNameMatechesSearch = event.title
        ?.trim()
        .toLocaleLowerCase()
        .includes(search.trim().toLocaleLowerCase());

      return eventNameMatechesSearch;
    });

    if (
      filters.filterUserCreatedEvents ||
      filters.filterUserParticipateEvents
    ) {
      const newFiltered = filtered.filter((event) => {
        let eventNameMatchesSearch = false;

        if (filters.filterUserCreatedEvents) {
          eventNameMatchesSearch = event.ownerId === loggedUser?.id;
          if (!eventNameMatchesSearch) return eventNameMatchesSearch;
        }
        if (filters.filterUserParticipateEvents) {
          const findVoluntary = event.volunteer?.filter(
            (voluntary) => loggedUser?.id === voluntary.id
          );
          eventNameMatchesSearch = !!findVoluntary && findVoluntary.length > 0;
        }

        return eventNameMatchesSearch;
      });

      setFilteredEvents(newFiltered);
    } else {
      setFilteredEvents(filtered);
    }
  };

  useEffect(() => {
    updateFilters();
  }, [search, filters, events]);

  const handleClick = (id: string | undefined) => {
    if (id) history.push(`/events/${id}`);
  };

  return (
    <>
      <Header variant="double" position="fixed" zIndex="99">
        <Box
          id="first-name"
          isRequired
          width="100%"
          bgColor="blue.200"
          mb="-1px"
        >
          <Box
            display="flex"
            justifyContent="center"
            alignItems="center"
            maxW="720px"
            mx="auto"
            p="8px 8px 0 8px"
          >
            <Image src={logoImage} w="96px" h="96px" />

            <SearchBar
              value={search}
              onChange={(e) => setSearch(e.target.value)}
            />
            <EventFilter />
          </Box>
        </Box>
      </Header>
      <Box
        h="calc(100vh - 50px)"
        overflowY="auto"
        pt={["160px", "220px"]}
        pb="10px"
      >
        {filteredEvents.length > 0 ? (
          filteredEvents.map((event: EventResponseListDTO) => (
            <Box key={event.id} onClick={() => handleClick(event.id)}>
              <ImageName event={event} />
              <Img
                w="100vw"
                h="250px"
                src={
                  event.images && event.images.length > 0
                    ? event.images[0]
                    : defaultImage
                }
                alt="imagem"
              />
              <EventInfo event={event} variant="primary" />
            </Box>
          ))
        ) : (
          <Box
            w="100%"
            h="50vh"
            display="flex"
            alignItems="center"
            justifyContent="center"
          >
            <Text>Nenhum evento encontrado.</Text>
          </Box>
        )}
      </Box>
      <BottomMenu defaultIndex={0} />
    </>
  );
};
export default ListEvent;
